@@ -10,6 +10,7 @@ from typing import TYPE_CHECKING, Any, Literal, cast
 from sqlspec.adapters.adbc._typing import AdbcSessionContext
 from sqlspec.adapters.adbc.core import (
     collect_rows,
+    create_mapped_exception,
     detect_dialect,
     driver_profile,
     get_statement_config,
@@ -18,7 +19,6 @@ from sqlspec.adapters.adbc.core import (
     normalize_postgres_empty_parameters,
     normalize_script_rowcount,
     prepare_postgres_parameters,
-    raise_exception,
     resolve_dialect_name,
     resolve_parameter_casts,
     resolve_rowcount,
@@ -90,12 +90,8 @@ class AdbcExceptionHandler:
         _ = exc_tb
         if exc_type is None:
             return False
-        try:
-            raise_exception(exc_val)
-        except Exception as mapped:
-            self.pending_exception = mapped
-            return True
-        return False
+        self.pending_exception = create_mapped_exception(exc_val)
+        return True
 
 
 class AdbcDriver(SyncDriverAdapterBase):
@@ -376,8 +372,11 @@ class AdbcDriver(SyncDriverAdapterBase):
         config = statement_config or self.statement_config
         prepared_statement = self.prepare_statement(statement, parameters, statement_config=config, kwargs=kwargs)
 
+        exc_handler = self.handle_database_exceptions()
+        arrow_result: ArrowResult | None = None
+
         # Use ADBC cursor for native Arrow
-        with self.with_cursor(self.connection) as cursor, self.handle_database_exceptions():
+        with self.with_cursor(self.connection) as cursor, exc_handler:
             if cursor is None:
                 msg = "Failed to create cursor"
                 raise DatabaseConnectionError(msg)
@@ -391,15 +390,22 @@ class AdbcDriver(SyncDriverAdapterBase):
             # Fetch as Arrow table (zero-copy!)
             arrow_table = cursor.fetch_arrow_table()
 
-            return build_arrow_result_from_table(
+            arrow_result = build_arrow_result_from_table(
                 prepared_statement,
                 arrow_table,
                 return_format=return_format,
                 batch_size=batch_size,
                 arrow_schema=arrow_schema,
             )
-        msg = "Unreachable"
-        raise RuntimeError(msg)  # pragma: no cover
+
+        if exc_handler.pending_exception is not None:
+            raise exc_handler.pending_exception from None
+
+        if arrow_result is None:
+            msg = "Unreachable"
+            raise RuntimeError(msg)  # pragma: no cover
+
+        return arrow_result
 
     # ─────────────────────────────────────────────────────────────────────────────
     # STORAGE API METHODS
@@ -444,8 +450,12 @@ class AdbcDriver(SyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         ingest_mode: Literal["append", "create", "replace", "create_append"]
         ingest_mode = "replace" if overwrite else "create_append"
-        with self.with_cursor(self.connection) as cursor, self.handle_database_exceptions():
+        exc_handler = self.handle_database_exceptions()
+        with self.with_cursor(self.connection) as cursor, exc_handler:
             cursor.adbc_ingest(table, arrow_table, mode=ingest_mode)
+
+        if exc_handler.pending_exception is not None:
+            raise exc_handler.pending_exception from None
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
         self._attach_partition_telemetry(telemetry_payload, partitioner)
