@@ -1,13 +1,14 @@
 """DuckDB adapter for SQLSpec."""
 
-from sqlspec.adapters.duckdb._types import DuckDBConnection
+from sqlspec.adapters.duckdb._typing import DuckDBConnection
 from sqlspec.adapters.duckdb.config import (
     DuckDBConfig,
     DuckDBConnectionParams,
     DuckDBExtensionConfig,
     DuckDBSecretConfig,
 )
-from sqlspec.adapters.duckdb.driver import DuckDBCursor, DuckDBDriver, DuckDBExceptionHandler, duckdb_statement_config
+from sqlspec.adapters.duckdb.core import default_statement_config
+from sqlspec.adapters.duckdb.driver import DuckDBCursor, DuckDBDriver, DuckDBExceptionHandler
 from sqlspec.adapters.duckdb.pool import DuckDBConnectionPool
 
 __all__ = (
@@ -20,5 +21,5 @@ __all__ = (
     "DuckDBExceptionHandler",
     "DuckDBExtensionConfig",
     "DuckDBSecretConfig",
-    "duckdb_statement_config",
+    "default_statement_config",
 )
