@@ -9,6 +9,6 @@ Notes:
     OLTP-optimized databases.
 """
 
-from sqlspec.adapters.duckdb.adk.store import DuckdbADKStore
+from sqlspec.adapters.duckdb.adk.store import DuckdbADKMemoryStore, DuckdbADKStore
 
-__all__ = ("DuckdbADKStore",)
+__all__ = ("DuckdbADKMemoryStore", "DuckdbADKStore")
