@@ -12,19 +12,23 @@ Notes:
     PostgreSQL or other OLTP-optimized databases.
 """
 
+import contextlib
 from datetime import datetime, timezone
-from typing import TYPE_CHECKING, Any, Final
+from typing import TYPE_CHECKING, Any, Final, cast
 
 from sqlspec.extensions.adk import BaseSyncADKStore, EventRecord, SessionRecord
+from sqlspec.extensions.adk.memory.store import BaseSyncADKMemoryStore
 from sqlspec.utils.logging import get_logger
 from sqlspec.utils.serializers import from_json, to_json
 
 if TYPE_CHECKING:
     from sqlspec.adapters.duckdb.config import DuckDBConfig
+    from sqlspec.extensions.adk import MemoryRecord
 
-logger = get_logger("adapters.duckdb.adk.store")
 
-__all__ = ("DuckdbADKStore",)
+__all__ = ("DuckdbADKMemoryStore", "DuckdbADKStore")
+
+logger = get_logger("sqlspec.adapters.duckdb.adk.store")
 
 DUCKDB_TABLE_NOT_FOUND_ERROR: Final = "does not exist"
 
@@ -58,7 +62,7 @@ class DuckdbADKStore(BaseSyncADKStore["DuckDBConfig"]):
             }
         )
         store = DuckdbADKStore(config)
-        store.create_tables()
+        store.ensure_tables()
 
         session = store.create_session(
             session_id="session-123",
@@ -183,7 +187,6 @@ class DuckdbADKStore(BaseSyncADKStore["DuckDBConfig"]):
         with self._config.provide_connection() as conn:
             conn.execute(self._get_create_sessions_table_sql())
             conn.execute(self._get_create_events_table_sql())
-        logger.debug("Created ADK tables: %s, %s", self._session_table, self._events_table)
 
     def create_session(
         self, session_id: str, app_name: str, user_id: str, state: "dict[str, Any]", owner_id: "Any | None" = None
@@ -561,3 +564,287 @@ class DuckdbADKStore(BaseSyncADKStore["DuckDBConfig"]):
             if DUCKDB_TABLE_NOT_FOUND_ERROR in str(e):
                 return []
             raise
+
+
+class DuckdbADKMemoryStore(BaseSyncADKMemoryStore["DuckDBConfig"]):
+    """DuckDB ADK memory store using synchronous DuckDB driver.
+
+    Implements memory entry storage for Google Agent Development Kit
+    using DuckDB's synchronous driver. Provides:
+    - Session memory storage with native JSON type
+    - Simple ILIKE search
+    - Native TIMESTAMP type support
+    - Deduplication via event_id unique constraint
+    - Efficient upserts using INSERT OR IGNORE
+    - Columnar storage for analytical queries
+
+    Args:
+        config: DuckDBConfig with extension_config["adk"] settings.
+
+    Example:
+        from sqlspec.adapters.duckdb import DuckDBConfig
+        from sqlspec.adapters.duckdb.adk.store import DuckdbADKMemoryStore
+
+        config = DuckDBConfig(
+            database="app.ddb",
+            extension_config={
+                "adk": {
+                    "memory_table": "adk_memory_entries",
+                    "memory_max_results": 20,
+                }
+            }
+        )
+        store = DuckdbADKMemoryStore(config)
+        store.ensure_tables()
+
+    Notes:
+        - Uses DuckDB native JSON type (not JSONB)
+        - TIMESTAMP for date/time storage with microsecond precision
+        - event_id UNIQUE constraint for deduplication
+        - Composite index on (app_name, user_id, timestamp DESC)
+        - Columnar storage provides excellent analytical query performance
+        - Optimized for OLAP workloads; for high-concurrency writes use PostgreSQL
+        - Configuration is read from config.extension_config["adk"]
+    """
+
+    __slots__ = ()
+
+    def __init__(self, config: "DuckDBConfig") -> None:
+        """Initialize DuckDB ADK memory store.
+
+        Args:
+            config: DuckDBConfig instance.
+
+        Notes:
+            Configuration is read from config.extension_config["adk"]:
+            - memory_table: Memory table name (default: "adk_memory_entries")
+            - memory_use_fts: Enable full-text search when supported (default: False)
+            - memory_max_results: Max search results (default: 20)
+            - owner_id_column: Optional owner FK column DDL (default: None)
+            - enable_memory: Whether memory is enabled (default: True)
+        """
+        super().__init__(config)
+
+    def _ensure_fts_extension(self, conn: Any) -> bool:
+        """Ensure the DuckDB FTS extension is available for this connection."""
+        with contextlib.suppress(Exception):
+            conn.execute("INSTALL fts")
+
+        try:
+            conn.execute("LOAD fts")
+        except Exception as exc:
+            logger.debug("DuckDB FTS extension unavailable: %s", exc)
+            return False
+
+        return True
+
+    def _create_fts_index(self, conn: Any) -> None:
+        """Create FTS index for the memory table."""
+        if not self._ensure_fts_extension(conn):
+            return
+
+        try:
+            conn.execute(f"PRAGMA create_fts_index('{self._memory_table}', 'id', 'content_text')")
+        except Exception as exc:
+            logger.debug("Failed to create DuckDB FTS index: %s", exc)
+
+    def _refresh_fts_index(self, conn: Any) -> None:
+        """Rebuild the FTS index to reflect recent changes."""
+        if not self._ensure_fts_extension(conn):
+            return
+
+        with contextlib.suppress(Exception):
+            conn.execute(f"PRAGMA drop_fts_index('{self._memory_table}')")
+
+        try:
+            conn.execute(f"PRAGMA create_fts_index('{self._memory_table}', 'id', 'content_text')")
+        except Exception as exc:
+            logger.debug("Failed to refresh DuckDB FTS index: %s", exc)
+
+    def _get_create_memory_table_sql(self) -> str:
+        """Get DuckDB CREATE TABLE SQL for memory entries.
+
+        Returns:
+            SQL statement to create memory table with indexes.
+        """
+        owner_id_line = ""
+        if self._owner_id_column_ddl:
+            owner_id_line = f",\n            {self._owner_id_column_ddl}"
+
+        return f"""
+        CREATE TABLE IF NOT EXISTS {self._memory_table} (
+            id VARCHAR(128) PRIMARY KEY,
+            session_id VARCHAR(128) NOT NULL,
+            app_name VARCHAR(128) NOT NULL,
+            user_id VARCHAR(128) NOT NULL,
+            event_id VARCHAR(128) NOT NULL UNIQUE,
+            author VARCHAR(256){owner_id_line},
+            timestamp TIMESTAMP NOT NULL,
+            content_json JSON NOT NULL,
+            content_text TEXT NOT NULL,
+            metadata_json JSON,
+            inserted_at TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP
+        );
+
+        CREATE INDEX IF NOT EXISTS idx_{self._memory_table}_app_user_time
+            ON {self._memory_table}(app_name, user_id, timestamp DESC);
+
+        CREATE INDEX IF NOT EXISTS idx_{self._memory_table}_session
+            ON {self._memory_table}(session_id);
+        """
+
+    def _get_drop_memory_table_sql(self) -> "list[str]":
+        """Get DuckDB DROP TABLE SQL statements."""
+        return [f"DROP TABLE IF EXISTS {self._memory_table}"]
+
+    def create_tables(self) -> None:
+        """Create the memory table and indexes if they don't exist."""
+        if not self._enabled:
+            return
+
+        with self._config.provide_connection() as conn:
+            conn.execute(self._get_create_memory_table_sql())
+            if self._use_fts:
+                self._create_fts_index(conn)
+
+    def insert_memory_entries(self, entries: "list[MemoryRecord]", owner_id: "object | None" = None) -> int:
+        """Bulk insert memory entries with deduplication."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        if not entries:
+            return 0
+
+        inserted_count = 0
+        if self._owner_id_column_name:
+            sql = f"""
+            INSERT INTO {self._memory_table} (
+                id, session_id, app_name, user_id, event_id, author,
+                {self._owner_id_column_name}, timestamp, content_json,
+                content_text, metadata_json, inserted_at
+            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
+            ON CONFLICT(event_id) DO NOTHING RETURNING 1
+            """
+        else:
+            sql = f"""
+            INSERT INTO {self._memory_table} (
+                id, session_id, app_name, user_id, event_id, author,
+                timestamp, content_json, content_text, metadata_json, inserted_at
+            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
+            ON CONFLICT(event_id) DO NOTHING RETURNING 1
+            """
+
+        with self._config.provide_connection() as conn:
+            for entry in entries:
+                params: tuple[Any, ...]
+                if self._owner_id_column_name:
+                    params = (
+                        entry["id"],
+                        entry["session_id"],
+                        entry["app_name"],
+                        entry["user_id"],
+                        entry["event_id"],
+                        entry["author"],
+                        owner_id,
+                        entry["timestamp"],
+                        to_json(entry["content_json"]),
+                        entry["content_text"],
+                        to_json(entry["metadata_json"]),
+                        entry["inserted_at"],
+                    )
+                else:
+                    params = (
+                        entry["id"],
+                        entry["session_id"],
+                        entry["app_name"],
+                        entry["user_id"],
+                        entry["event_id"],
+                        entry["author"],
+                        entry["timestamp"],
+                        to_json(entry["content_json"]),
+                        entry["content_text"],
+                        to_json(entry["metadata_json"]),
+                        entry["inserted_at"],
+                    )
+                result = conn.execute(sql, params)
+                inserted_count += len(result.fetchall())
+            conn.commit()
+        return inserted_count
+
+    def search_entries(
+        self, query: str, app_name: str, user_id: str, limit: "int | None" = None
+    ) -> "list[MemoryRecord]":
+        """Search memory entries by text query."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        if not query:
+            return []
+
+        limit_value = limit or self._max_results
+        if self._use_fts:
+            sql = f"""
+            SELECT * FROM {self._memory_table}
+            WHERE app_name = ? AND user_id = ? AND content_text @@ ?
+            ORDER BY timestamp DESC
+            LIMIT ?
+            """
+            params = (app_name, user_id, query, limit_value)
+        else:
+            sql = f"""
+            SELECT * FROM {self._memory_table}
+            WHERE app_name = ? AND user_id = ? AND content_text ILIKE ?
+            ORDER BY timestamp DESC
+            LIMIT ?
+            """
+            params = (app_name, user_id, f"%{query}%", limit_value)
+
+        with self._config.provide_connection() as conn:
+            rows = conn.execute(sql, params).fetchall()
+            columns = [col[0] for col in conn.description or []]
+        records: list[MemoryRecord] = []
+        for row in rows:
+            record = cast("MemoryRecord", dict(zip(columns, row, strict=False)))
+            content_value = record["content_json"]
+            if isinstance(content_value, (str, bytes)):
+                record["content_json"] = from_json(content_value)
+            metadata_value = record.get("metadata_json")
+            if isinstance(metadata_value, (str, bytes)):
+                record["metadata_json"] = from_json(metadata_value)
+            records.append(record)
+        if self._use_fts:
+            with self._config.provide_connection() as conn:
+                self._refresh_fts_index(conn)
+        return records
+
+    def delete_entries_by_session(self, session_id: str) -> int:
+        """Delete all memory entries for a specific session."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        sql = f"DELETE FROM {self._memory_table} WHERE session_id = ? RETURNING 1"
+        with self._config.provide_connection() as conn:
+            result = conn.execute(sql, (session_id,))
+            deleted_count = len(result.fetchall())
+            conn.commit()
+            return deleted_count
+
+    def delete_entries_older_than(self, days: int) -> int:
+        """Delete memory entries older than specified days."""
+        if not self._enabled:
+            msg = "Memory store is disabled"
+            raise RuntimeError(msg)
+
+        sql = f"""
+        DELETE FROM {self._memory_table}
+        WHERE inserted_at < (CURRENT_TIMESTAMP - INTERVAL '{days} days')
+        RETURNING 1
+        """
+        with self._config.provide_connection() as conn:
+            result = conn.execute(sql)
+            deleted_count = len(result.fetchall())
+            conn.commit()
+            return deleted_count
