@@ -22,12 +22,12 @@ from sqlspec.adapters.oracledb.core import (
     coerce_large_string_parameters_sync,
     collect_async_rows,
     collect_sync_rows,
+    create_mapped_exception,
     default_statement_config,
     driver_profile,
     normalize_column_names,
     normalize_execute_many_parameters_async,
     normalize_execute_many_parameters_sync,
-    raise_exception,
     resolve_rowcount,
 )
 from sqlspec.adapters.oracledb.data_dictionary import OracledbAsyncDataDictionary, OracledbSyncDataDictionary
@@ -286,11 +286,8 @@ class OracleSyncExceptionHandler:
         if exc_type is None:
             return False
         if issubclass(exc_type, oracledb.DatabaseError):
-            try:
-                raise_exception(exc_val)
-            except Exception as mapped:
-                self.pending_exception = mapped
-                return True
+            self.pending_exception = create_mapped_exception(exc_val)
+            return True
         return False
 
 
@@ -318,11 +315,8 @@ class OracleAsyncExceptionHandler:
         if exc_type is None:
             return False
         if issubclass(exc_type, oracledb.DatabaseError):
-            try:
-                raise_exception(exc_val)
-            except Exception as mapped:
-                self.pending_exception = mapped
-                return True
+            self.pending_exception = create_mapped_exception(exc_val)
+            return True
         return False
 
 
@@ -640,13 +634,19 @@ class OracleSyncDriver(OraclePipelineMixin, SyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         if overwrite:
             statement = build_truncate_statement(table)
-            with self.handle_database_exceptions():
+            exc_handler = self.handle_database_exceptions()
+            with exc_handler:
                 self.connection.execute(statement)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         columns, records = self._arrow_table_to_rows(arrow_table)
         if records:
             statement = build_insert_statement(table, columns)
-            with self.with_cursor(self.connection) as cursor, self.handle_database_exceptions():
+            exc_handler = self.handle_database_exceptions()
+            with self.with_cursor(self.connection) as cursor, exc_handler:
                 cursor.executemany(statement, records)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
         self._attach_partition_telemetry(telemetry_payload, partitioner)
@@ -1093,13 +1093,19 @@ class OracleAsyncDriver(OraclePipelineMixin, AsyncDriverAdapterBase):
         arrow_table = self._coerce_arrow_table(source)
         if overwrite:
             statement = build_truncate_statement(table)
-            async with self.handle_database_exceptions():
+            exc_handler = self.handle_database_exceptions()
+            async with exc_handler:
                 await self.connection.execute(statement)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         columns, records = self._arrow_table_to_rows(arrow_table)
         if records:
             statement = build_insert_statement(table, columns)
-            async with self.with_cursor(self.connection) as cursor, self.handle_database_exceptions():
+            exc_handler = self.handle_database_exceptions()
+            async with self.with_cursor(self.connection) as cursor, exc_handler:
                 await cursor.executemany(statement, records)
+            if exc_handler.pending_exception is not None:
+                raise exc_handler.pending_exception from None
         telemetry_payload = self._build_ingest_telemetry(arrow_table)
         telemetry_payload["destination"] = table
         self._attach_partition_telemetry(telemetry_payload, partitioner)
