@@ -4,19 +4,45 @@ from datetime import datetime, timedelta, timezone
 from typing import TYPE_CHECKING
 
 from sqlspec.extensions.litestar.store import BaseSQLSpecStore
-from sqlspec.utils.logging import get_logger
 from sqlspec.utils.sync_tools import async_
+from sqlspec.utils.type_guards import is_async_readable, is_readable
 
 if TYPE_CHECKING:
     from sqlspec.adapters.oracledb.config import OracleAsyncConfig, OracleSyncConfig
 
-logger = get_logger("adapters.oracledb.litestar.store")
 
 ORACLE_SMALL_BLOB_LIMIT = 32000
 
 __all__ = ("OracleAsyncStore", "OracleSyncStore")
 
 
+def _coerce_bytes_payload(value: object) -> bytes:
+    """Coerce a payload into bytes for session storage."""
+    if value is None:
+        return b""
+    if isinstance(value, bytes):
+        return value
+    if isinstance(value, str):
+        return value.encode("utf-8")
+    return str(value).encode("utf-8")
+
+
+async def _read_blob_async(value: object) -> bytes:
+    """Read LOB values from async connections into bytes."""
+    if is_async_readable(value):
+        return _coerce_bytes_payload(await value.read())
+    if is_readable(value):
+        return _coerce_bytes_payload(value.read())
+    return _coerce_bytes_payload(value)
+
+
+def _read_blob_sync(value: object) -> bytes:
+    """Read LOB values from sync connections into bytes."""
+    if is_readable(value):
+        return _coerce_bytes_payload(value.read())
+    return _coerce_bytes_payload(value)
+
+
 class OracleAsyncStore(BaseSQLSpecStore["OracleAsyncConfig"]):
     """Oracle session store using async OracleDB driver.
 
@@ -36,7 +62,7 @@ class OracleAsyncStore(BaseSQLSpecStore["OracleAsyncConfig"]):
         from sqlspec.adapters.oracledb.litestar.store import OracleAsyncStore
 
         config = OracleAsyncConfig(
-            pool_config={"dsn": "oracle://..."},
+            connection_config={"dsn": "oracle://..."},
             extension_config={
                 "litestar": {
                     "session_table": "my_sessions",
@@ -155,7 +181,7 @@ class OracleAsyncStore(BaseSQLSpecStore["OracleAsyncConfig"]):
         async with self._config.provide_session() as driver:
             await driver.execute_script(sql)
 
-        logger.debug("Created session table: %s", self._table_name)
+        self._log_table_created()
 
     async def get(self, key: str, renew_for: "int | timedelta | None" = None) -> "bytes | None":
         """Get a session value by key.
@@ -199,11 +225,7 @@ class OracleAsyncStore(BaseSQLSpecStore["OracleAsyncConfig"]):
                     await cursor.execute(update_sql, {"expires_at": new_expires_at, "session_id": key})
                     await conn.commit()
 
-            try:
-                blob_data = await data_blob.read()
-                return bytes(blob_data) if blob_data is not None else bytes(data_blob)
-            except AttributeError:
-                return bytes(data_blob)
+            return await _read_blob_async(data_blob)
 
     async def set(self, key: str, value: "str | bytes", expires_in: "int | timedelta | None" = None) -> None:
         """Store a session value.
@@ -292,7 +314,7 @@ class OracleAsyncStore(BaseSQLSpecStore["OracleAsyncConfig"]):
             cursor = conn.cursor()
             await cursor.execute(sql)
             await conn.commit()
-        logger.debug("Deleted all sessions from table: %s", self._table_name)
+        self._log_delete_all()
 
     async def exists(self, key: str) -> bool:
         """Check if a session key exists and is not expired.
@@ -374,7 +396,7 @@ class OracleAsyncStore(BaseSQLSpecStore["OracleAsyncConfig"]):
             count = cursor.rowcount if cursor.rowcount is not None else 0
             await conn.commit()
             if count > 0:
-                logger.debug("Cleaned up %d expired sessions", count)
+                self._log_delete_expired(count)
             return count
 
 
@@ -404,7 +426,7 @@ class OracleSyncStore(BaseSQLSpecStore["OracleSyncConfig"]):
         from sqlspec.adapters.oracledb.litestar.store import OracleSyncStore
 
         config = OracleSyncConfig(
-            pool_config={"dsn": "oracle://..."},
+            connection_config={"dsn": "oracle://..."},
             extension_config={
                 "litestar": {
                     "session_table": "my_sessions",
@@ -523,7 +545,7 @@ class OracleSyncStore(BaseSQLSpecStore["OracleSyncConfig"]):
         with self._config.provide_session() as driver:
             driver.execute_script(sql)
 
-        logger.debug("Created session table: %s", self._table_name)
+        self._log_table_created()
 
     async def create_table(self) -> None:
         """Create the session table if it doesn't exist."""
@@ -562,13 +584,7 @@ class OracleSyncStore(BaseSQLSpecStore["OracleSyncConfig"]):
                     cursor.execute(update_sql, {"expires_at": new_expires_at, "session_id": key})
                     conn.commit()
 
-            try:
-                if hasattr(data_blob, "read"):
-                    blob_data = data_blob.read()
-                    return bytes(blob_data) if blob_data is not None else bytes(data_blob)
-                return bytes(data_blob)
-            except AttributeError:
-                return bytes(data_blob)
+            return _read_blob_sync(data_blob)
 
     async def get(self, key: str, renew_for: "int | timedelta | None" = None) -> "bytes | None":
         """Get a session value by key.
@@ -673,7 +689,7 @@ class OracleSyncStore(BaseSQLSpecStore["OracleSyncConfig"]):
             cursor = conn.cursor()
             cursor.execute(sql)
             conn.commit()
-        logger.debug("Deleted all sessions from table: %s", self._table_name)
+        self._log_delete_all()
 
     async def delete_all(self) -> None:
         """Delete all sessions from the store."""
@@ -753,7 +769,7 @@ class OracleSyncStore(BaseSQLSpecStore["OracleSyncConfig"]):
             count = cursor.rowcount if cursor.rowcount is not None else 0
             conn.commit()
             if count > 0:
-                logger.debug("Cleaned up %d expired sessions", count)
+                self._log_delete_expired(count)
             return count
 
     async def delete_expired(self) -> int:
