@@ -4,7 +4,6 @@ Extends Starlette integration with dependency injection helpers for FastAPI's
 Depends() system, including filter dependency builders.
 """
 
-from sqlspec.config import StarletteConfig
 from sqlspec.extensions.fastapi.extension import SQLSpecPlugin
 from sqlspec.extensions.fastapi.providers import DependencyDefaults, FieldNameType, FilterConfig, provide_filters
 from sqlspec.extensions.starlette.middleware import SQLSpecAutocommitMiddleware, SQLSpecManualMiddleware
@@ -16,6 +15,5 @@ __all__ = (
     "SQLSpecAutocommitMiddleware",
     "SQLSpecManualMiddleware",
     "SQLSpecPlugin",
-    "StarletteConfig",
     "provide_filters",
 )
