@@ -1,14 +1,23 @@
 """Flask extension for SQLSpec database integration."""
 
 import atexit
+import logging
 from typing import TYPE_CHECKING, Any, Literal
 
 from sqlspec.base import SQLSpec
 from sqlspec.config import AsyncDatabaseConfig, NoPoolAsyncConfig
+from sqlspec.core import CorrelationExtractor
 from sqlspec.exceptions import ImproperConfigurationError
 from sqlspec.extensions.flask._state import FlaskConfigState
-from sqlspec.extensions.flask._utils import get_or_create_session
-from sqlspec.utils.logging import get_logger
+from sqlspec.extensions.flask._utils import (
+    get_context_value,
+    get_or_create_session,
+    has_context_value,
+    pop_context_value,
+    set_context_value,
+)
+from sqlspec.utils.correlation import CorrelationContext
+from sqlspec.utils.logging import get_logger, log_with_context
 from sqlspec.utils.portal import PortalProvider
 
 if TYPE_CHECKING:
@@ -16,7 +25,7 @@ if TYPE_CHECKING:
 
 __all__ = ("SQLSpecPlugin",)
 
-logger = get_logger("extensions.flask")
+logger = get_logger("sqlspec.extensions.flask")
 
 DEFAULT_COMMIT_MODE: Literal["manual"] = "manual"
 DEFAULT_SESSION_KEY = "db_session"
@@ -36,7 +45,7 @@ class SQLSpecPlugin:
 
         sqlspec = SQLSpec()
         config = SqliteConfig(
-            pool_config={"database": "app.db"},
+            connection_config={"database": "app.db"},
             extension_config={
                 "flask": {
                     "commit_mode": "autocommit",
@@ -69,6 +78,8 @@ class SQLSpecPlugin:
         self._has_async_configs = False
         self._cleanup_registered = False
         self._shutdown_complete = False
+        self._enable_correlation = False
+        self._extractor: CorrelationExtractor | None = None
 
         for cfg in self._sqlspec.configs.values():
             state = self._create_config_state(cfg)
@@ -77,6 +88,14 @@ class SQLSpecPlugin:
             if state.is_async:
                 self._has_async_configs = True
 
+            if state.enable_correlation_middleware and not self._enable_correlation:
+                self._enable_correlation = True
+                self._extractor = CorrelationExtractor(
+                    primary_header=state.correlation_header,
+                    additional_headers=state.correlation_headers,
+                    auto_trace_headers=state.auto_trace_headers,
+                )
+
         if app is not None:
             self.init_app(app)
 
@@ -98,6 +117,13 @@ class SQLSpecPlugin:
         extra_rollback_statuses = flask_config.get("extra_rollback_statuses")
         disable_di = flask_config.get("disable_di", False)
 
+        enable_correlation = flask_config.get("enable_correlation_middleware", False)
+        correlation_header = flask_config.get("correlation_header", "x-request-id")
+        correlation_headers = flask_config.get("correlation_headers")
+        if correlation_headers is not None:
+            correlation_headers = tuple(correlation_headers)
+        auto_trace_headers = flask_config.get("auto_trace_headers", True)
+
         is_async = isinstance(config, (AsyncDatabaseConfig, NoPoolAsyncConfig))
 
         return FlaskConfigState(
@@ -109,6 +135,10 @@ class SQLSpecPlugin:
             extra_rollback_statuses=extra_rollback_statuses,
             is_async=is_async,
             disable_di=disable_di,
+            enable_correlation_middleware=enable_correlation,
+            correlation_header=correlation_header,
+            correlation_headers=correlation_headers,
+            auto_trace_headers=auto_trace_headers,
         )
 
     def init_app(self, app: "Flask") -> None:
@@ -132,7 +162,7 @@ class SQLSpecPlugin:
         if self._has_async_configs:
             self._portal = PortalProvider()
             self._portal.start()
-            logger.debug("Portal provider started for async adapters")
+            log_with_context(logger, logging.DEBUG, "extension.init", framework="flask", stage="portal_started")
 
         pools: dict[str, Any] = {}
         for config_state in self._config_states:
@@ -142,6 +172,9 @@ class SQLSpecPlugin:
                 else:
                     pool = config_state.config.create_pool()
                 pools[config_state.session_key] = pool
+                log_with_context(
+                    logger, logging.DEBUG, "session.create", framework="flask", session_key=config_state.session_key
+                )
 
         app.extensions["sqlspec"] = {"plugin": self, "pools": pools}
 
@@ -152,7 +185,15 @@ class SQLSpecPlugin:
 
         self._register_shutdown_hook()
 
-        logger.debug("SQLSpec Flask extension initialized")
+        log_with_context(
+            logger,
+            logging.DEBUG,
+            "extension.init",
+            framework="flask",
+            stage="configured",
+            config_count=len(self._config_states),
+            async_enabled=self._has_async_configs,
+        )
 
     def _validate_unique_keys(self) -> None:
         """Validate that all state keys are unique across configs.
@@ -186,8 +227,14 @@ class SQLSpecPlugin:
 
         Stores connection in Flask g object for each configured database.
         Also stores context managers for proper cleanup.
+        Extracts correlation ID if correlation middleware is enabled.
         """
-        from flask import current_app, g
+        from flask import current_app, g, request
+
+        if self._enable_correlation and self._extractor is not None:
+            correlation_id = self._extractor.extract(lambda h: request.headers.get(h))
+            set_context_value(g, "correlation_id", correlation_id)
+            CorrelationContext.set(correlation_id)
 
         for config_state in self._config_states:
             if config_state.disable_di:
@@ -202,13 +249,13 @@ class SQLSpecPlugin:
                 else:
                     connection = conn_ctx.__enter__()  # type: ignore[union-attr]
 
-                setattr(g, f"{config_state.connection_key}_ctx", conn_ctx)
+                set_context_value(g, f"{config_state.connection_key}_ctx", conn_ctx)
             elif config_state.is_async:
                 connection = self._portal.portal.call(config_state.config.create_connection)  # type: ignore[union-attr,arg-type]
             else:
                 connection = config_state.config.create_connection()
 
-            setattr(g, config_state.connection_key, connection)
+            set_context_value(g, config_state.connection_key, connection)
 
     def _after_request_handler(self, response: "Response") -> "Response":
         """Handle transaction after request based on response status.
@@ -217,10 +264,15 @@ class SQLSpecPlugin:
             response: Flask response object.
 
         Returns:
-            Response object (unchanged).
+            Response object with correlation ID header if enabled.
         """
         from flask import g
 
+        if self._enable_correlation:
+            correlation_id = get_context_value(g, "correlation_id", None)
+            if correlation_id:
+                response.headers["X-Correlation-ID"] = correlation_id
+
         for config_state in self._config_states:
             if config_state.disable_di:
                 continue
@@ -229,7 +281,7 @@ class SQLSpecPlugin:
                 continue
 
             cache_key = f"sqlspec_session_cache_{config_state.session_key}"
-            session = getattr(g, cache_key, None)
+            session = get_context_value(g, cache_key, None)
 
             if session is None:
                 continue
@@ -244,20 +296,25 @@ class SQLSpecPlugin:
     def _teardown_appcontext_handler(self, _exc: "BaseException | None" = None) -> None:
         """Clean up connections when request context ends.
 
-        Closes all connections and cleans up g object.
+        Closes all connections, cleans up g object, and clears correlation context.
 
         Args:
             _exc: Exception that occurred (if any).
         """
         from flask import g
 
+        if self._enable_correlation:
+            CorrelationContext.clear()
+            if has_context_value(g, "correlation_id"):
+                pop_context_value(g, "correlation_id")
+
         for config_state in self._config_states:
             if config_state.disable_di:
                 continue
 
-            connection = getattr(g, config_state.connection_key, None)
+            connection = get_context_value(g, config_state.connection_key, None)
             ctx_key = f"{config_state.connection_key}_ctx"
-            conn_ctx = getattr(g, ctx_key, None)
+            conn_ctx = get_context_value(g, ctx_key, None)
 
             if connection is not None:
                 try:
@@ -270,17 +327,26 @@ class SQLSpecPlugin:
                         self._portal.portal.call(connection.close)  # type: ignore[union-attr]
                     else:
                         connection.close()
-                except Exception:
-                    logger.exception("Error closing connection")
+                except Exception as exc:
+                    log_with_context(
+                        logger,
+                        logging.ERROR,
+                        "session.close",
+                        framework="flask",
+                        session_key=config_state.session_key,
+                        operation="connection",
+                        status="failed",
+                        error_type=type(exc).__name__,
+                    )
 
-                if hasattr(g, config_state.connection_key):
-                    delattr(g, config_state.connection_key)
-                if hasattr(g, ctx_key):
-                    delattr(g, ctx_key)
+                if has_context_value(g, config_state.connection_key):
+                    pop_context_value(g, config_state.connection_key)
+                if has_context_value(g, ctx_key):
+                    pop_context_value(g, ctx_key)
 
             cache_key = f"sqlspec_session_cache_{config_state.session_key}"
-            if hasattr(g, cache_key):
-                delattr(g, cache_key)
+            if has_context_value(g, cache_key):
+                pop_context_value(g, cache_key)
 
     def get_session(self, key: "str | None" = None) -> Any:
         """Get or create database session for current request.
@@ -310,7 +376,7 @@ class SQLSpecPlugin:
 
         config_state = self._config_states[0] if key is None else self._get_config_state_by_key(key)
 
-        return getattr(g, config_state.connection_key)
+        return get_context_value(g, config_state.connection_key)
 
     def _get_config_state_by_key(self, key: str) -> FlaskConfigState:
         """Get config state by session key.
@@ -346,8 +412,16 @@ class SQLSpecPlugin:
         if self._portal is not None:
             try:
                 self._portal.stop()
-            except Exception:
-                logger.exception("Error stopping portal during shutdown")
+            except Exception as exc:
+                log_with_context(
+                    logger,
+                    logging.ERROR,
+                    "extension.init",
+                    framework="flask",
+                    stage="shutdown",
+                    status="failed",
+                    error_type=type(exc).__name__,
+                )
             finally:
                 self._portal = None
 
@@ -357,15 +431,40 @@ class SQLSpecPlugin:
         try:
             if config_state.is_async:
                 if self._portal is None:
-                    logger.debug(
-                        "Portal not initialized - skipping async pool shutdown for %s", config_state.session_key
+                    log_with_context(
+                        logger,
+                        logging.DEBUG,
+                        "session.close",
+                        framework="flask",
+                        session_key=config_state.session_key,
+                        operation="pool",
+                        status="skipped",
+                        reason="portal_not_initialized",
                     )
                     return
                 _ = self._portal.portal.call(config_state.config.close_pool)  # type: ignore[arg-type]
             else:
                 config_state.config.close_pool()
-        except Exception:
-            logger.exception("Error closing pool during shutdown for key %s", config_state.session_key)
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "session.close",
+                framework="flask",
+                session_key=config_state.session_key,
+                operation="pool",
+                status="complete",
+            )
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "session.close",
+                framework="flask",
+                session_key=config_state.session_key,
+                operation="pool",
+                status="failed",
+                error_type=type(exc).__name__,
+            )
 
     def _execute_commit(self, session: Any, config_state: FlaskConfigState) -> None:
         """Execute commit on session.
@@ -381,8 +480,17 @@ class SQLSpecPlugin:
             else:
                 connection = self.get_connection(config_state.session_key)
                 connection.commit()
-        except Exception:
-            logger.exception("Error committing transaction")
+        except Exception as exc:
+            log_with_context(
+                logger,
+                logging.ERROR,
+                "session.close",
+                framework="flask",
+                session_key=config_state.session_key,
+                operation="commit",
+                status="failed",
+                error_type=type(exc).__name__,
+            )
 
     def _execute_rollback(self, session: Any, config_state: FlaskConfigState) -> None:
         """Execute rollback on session.
@@ -399,4 +507,13 @@ class SQLSpecPlugin:
                 connection = self.get_connection(config_state.session_key)
                 connection.rollback()
         except Exception as exc:
-            logger.debug("Rollback failed (may be no active transaction): %s", exc)
+            log_with_context(
+                logger,
+                logging.DEBUG,
+                "session.close",
+                framework="flask",
+                session_key=config_state.session_key,
+                operation="rollback",
+                status="failed",
+                error_type=type(exc).__name__,
+            )
