@@ -2,7 +2,6 @@
 
 import importlib
 import inspect
-import logging
 import os
 import subprocess
 from datetime import datetime, timezone
@@ -10,6 +9,7 @@ from pathlib import Path
 from typing import TYPE_CHECKING, Any, cast
 
 from sqlspec.migrations.templates import MigrationTemplateSettings, TemplateValidationError, build_template_settings
+from sqlspec.utils.logging import get_logger
 from sqlspec.utils.text import slugify
 
 if TYPE_CHECKING:
@@ -20,7 +20,7 @@ if TYPE_CHECKING:
 
 __all__ = ("create_migration_file", "drop_all", "get_author")
 
-logger = logging.getLogger(__name__)
+logger = get_logger(__name__)
 
 
 def create_migration_file(
@@ -34,7 +34,7 @@ def create_migration_file(
 ) -> Path:
     """Create a new migration file from template."""
 
-    migration_config = getattr(config, "migration_config", {}) or {}
+    migration_config = cast("dict[str, Any]", config.migration_config) if config is not None else {}
     settings = template_settings or build_template_settings(migration_config)
     author = get_author(migration_config.get("author"), config=config)
     safe_message = _slugify_message(message)
@@ -174,7 +174,7 @@ def _resolve_author_callable(import_path: str, config: "DatabaseConfigProtocol[A
     if not module_name or not attr_name:
         _raise_callable_error("Callable author path must be in 'module:function' format")
     module = importlib.import_module(module_name)
-    candidate_obj = getattr(module, attr_name, None)
+    candidate_obj = module.__dict__.get(attr_name)
     if candidate_obj is None or not callable(candidate_obj):
         _raise_callable_error(f"Callable '{import_path}' is not callable")
     candidate = cast("Callable[..., Any]", candidate_obj)
@@ -230,8 +230,8 @@ def _derive_project_slug(config: "DatabaseConfigProtocol[Any, Any, Any] | None")
 def _resolve_adapter_name(config: "DatabaseConfigProtocol[Any, Any, Any] | None") -> str:
     if config is None:
         return "UnknownAdapter"
-    driver_type = getattr(config, "driver_type", None)
-    if driver_type is not None and hasattr(driver_type, "__name__"):
+    driver_type = config.driver_type
+    if driver_type is not None:
         return str(driver_type.__name__)
     return type(config).__name__
 
